@@ -35,8 +35,9 @@
 
 import json
 import unittest
+from binascii import unhexlify
 
-from Cryptodome.Util.py3compat import tobytes, _memoryview, unhexlify
+from Cryptodome.Util.py3compat import tobytes, _memoryview
 
 from Cryptodome.Hash import CMAC
 from Cryptodome.Cipher import AES, DES3
@@ -45,6 +46,7 @@ from Cryptodome.Hash import SHAKE128
 from Cryptodome.Util._file_system import pycryptodome_filename
 from Cryptodome.Util.strxor import strxor
 
+from Cryptodome.SelfTest.st_common import list_test_cases
 
 # This is a list of (key, data, result, description, module) tuples.
 test_data = [
@@ -249,10 +251,10 @@ def get_tag_random(tag, length):
     return SHAKE128.new(data=tobytes(tag)).read(length)
 
 
-class MultipleUpdates(unittest.TestCase):
-    """Verify that internal caching is implemented correctly"""
+class TestCMAC(unittest.TestCase):
 
-    def runTest(self):
+    def test_internal_caching(self):
+        """Verify that internal caching is implemented correctly"""
 
         data_to_mac = get_tag_random("data_to_mac", 128)
         key = get_tag_random("key", 16)
@@ -270,6 +272,24 @@ class MultipleUpdates(unittest.TestCase):
                 mac.update(chunk)
             self.assertEqual(ref_mac, mac.digest())
 
+    def test_update_after_digest(self):
+        msg = b"rrrrttt"
+        key = b"4" * 16
+
+        # Normally, update() cannot be done after digest()
+        h = CMAC.new(key, msg[:4], ciphermod=AES)
+        dig1 = h.digest()
+        self.assertRaises(TypeError, h.update, msg[4:])
+        dig2 = CMAC.new(key, msg, ciphermod=AES).digest()
+
+        # With the proper flag, it is allowed
+        h2 = CMAC.new(key, msg[:4], ciphermod=AES, update_after_digest=True)
+        self.assertEquals(h2.digest(), dig1)
+        # ... and the subsequent digest applies to the entire message
+        # up to that point
+        h2.update(msg[4:])
+        self.assertEquals(h2.digest(), dig2)
+
 
 class ByteArrayTests(unittest.TestCase):
 
@@ -346,10 +366,9 @@ class TestVectorsWycheproof(unittest.TestCase):
         self._id = "None"
 
     def setUp(self):
-        file_in = open(pycryptodome_filename(
-                        "Cryptodome.SelfTest.Hash.test_vectors.wycheproof".split("."),
-                        "aes_cmac_test.json"), "rt")
-        tv_tree = json.load(file_in)
+        comps = "Cryptodome.SelfTest.Hash.test_vectors.wycheproof".split(".")
+        with open(pycryptodome_filename(comps, "aes_cmac_test.json"), "rt") as file_in:
+            tv_tree = json.load(file_in)
 
         class TestVector(object):
             pass
@@ -382,7 +401,7 @@ class TestVectorsWycheproof(unittest.TestCase):
         
         try:
             tag = CMAC.new(tv.key, tv.msg, ciphermod=AES, mac_len=tv.tag_size).digest()
-        except ValueError, e:
+        except ValueError as e:
             if len(tv.key) not in (16, 24, 32) and "key length" in str(e):
                 return
             raise e
@@ -395,7 +414,7 @@ class TestVectorsWycheproof(unittest.TestCase):
        
         try:
             mac = CMAC.new(tv.key, tv.msg, ciphermod=AES, mac_len=tv.tag_size)
-        except ValueError, e:
+        except ValueError as e:
             if len(tv.key) not in (16, 24, 32) and "key length" in str(e):
                 return
             raise e
@@ -417,7 +436,7 @@ class TestVectorsWycheproof(unittest.TestCase):
 def get_tests(config={}):
     global test_data
     import types
-    from common import make_mac_tests
+    from .common import make_mac_tests
     
     wycheproof_warnings = config.get('wycheproof_warnings')
 
@@ -429,10 +448,13 @@ def get_tests(config={}):
         params_test_data.append(t)
 
     tests = make_mac_tests(CMAC, "CMAC", params_test_data)
-    tests.append(MultipleUpdates())
     tests.append(ByteArrayTests())
-    if _memoryview is not types.NoneType:
+    tests.append(list_test_cases(TestCMAC))
+    
+    import sys
+    if sys.version[:3] != "2.6":
         tests.append(MemoryViewTests())
+    
     tests += [ TestVectorsWycheproof(wycheproof_warnings) ]
     return tests
 
